import * as d3 from "d3";
import * as R from "ramda";
import { EventEmitter } from "../util";
export class Backend extends EventEmitter {
    registeredViews = [];
    constructor() {
        super();
        this.on("data", (table, ...args) => {
            // TODO: if result didn't change, don't need to call render
            this.registeredViews.forEach(({ view, postprocessFuncs }) => {
                if (R.isNil(view.render))
                    return;
                let pipe = R.pipe(...postprocessFuncs);
                let newTable = pipe(table);
                view.render(newTable, ...args);
            });
        });
    }
    // Used in case there's some asynchronous initialization
    // needed to set up backend.  For instance, DuckDB 
    // needs to spin up workers and possibly fetch data files.
    async init() { }
    // @param view view must have a .render() function that takes
    //       a table:Array<object> as input 
    //
    // @param postprocessFuncs function or List of functions to post process 
    //        execution result before emitting event.  
    //        Useful for applying mappings before sending data to a View.
    registerView(view, postprocessFuncs) {
        console.log("Backend.registerView", view, postprocessFuncs);
        postprocessFuncs = R.flatten([postprocessFuncs ?? R.identity]);
        this.registeredViews.push({ view, postprocessFuncs });
    }
    addPostprocessing(viewId, f) {
        this.registeredViews.forEach(({ view, postprocessFuncs }) => {
            if (view.spec?.id == viewId)
                postprocessFuncs.push(f);
        });
    }
    // @deprecated
    merge(q, newq) {
        return newq;
    }
    // After executing the query, the backend should emit
    // a "data" event with the result value
    //
    // @param q query that the backend should execute.  
    //    May be a query string, or a binding (for difftrees), 
    //    or some other representation.
    //
    // @param name optional event name to emit once results
    //    are ready
    //
    async execute(q, name) {
    }
}
