import * as R from "ramda";
import { DTType, DiffTreeSpec } from "./dtspec";
let rmEmptyStrs = R.filter((s) => s.length > 0);
export class DiffTree {
    binding = {};
    spec = {};
    constructor(spec) {
        this.spec = spec;
    }
    update(newBinding) {
        this.binding = R.mergeRight(this.binding, newBinding);
        return this;
    }
    toSQL() {
        return toSQL(this.binding, this.spec);
    }
}
//
// @param node: difftree spec for current difftree node
// @param binding: dict of choicenodeid -> binding value
//   a multi choice node maps to a list of subbindings
//   binding can be nested at most one level for now
//
function toSQL(binding, node) {
    let children = node.children ?? [];
    let applyBinding = R.partial(toSQL, [binding]);
    let txts = [];
    let idx;
    switch (node?.type) {
        case DTType.LITERAL:
            return node.value;
        case DTType.AST:
            txts = children.map(applyBinding);
            return rmEmptyStrs(txts).join("");
        case DTType.LIST:
            txts = children.map(applyBinding);
            return rmEmptyStrs(txts).join(node.delim);
        case DTType.ANY:
            idx = binding[node.id] ?? node.default ?? 0;
            if (idx == "inf")
                return "9e999";
            if (idx == "-inf")
                return "-9e999";
            //
            // XXX: this implicit coersion is sloppy.  
            // Treating Any as a VAL node should be explicit
            //
            // We need to rethink how ANY should be bound
            //
            let n = +idx;
            if (Number.isNaN(n))
                return `'${idx}'`;
            if (idx < 0 || idx >= children.length)
                return n;
            return applyBinding(children[n]);
        case DTType.OPT:
            idx = binding[node.id] ?? node.default ?? 0;
            if (R.includes(idx, [1, "selected"]))
                return applyBinding(children[0]);
            return "";
        // XXX: magically sets itself if a descendant opt is set??
        //case "co-optional":
        //  return (params) => {
        //    if (optional_descendent_set(node, params)) 
        //      return children[0](params)
        //    return "";
        //  }
        case DTType.FIXEDMULTI:
        case DTType.MULTI:
            let childBindings = binding[node.id] ?? node.default ?? [];
            if (!R.is(Array, childBindings))
                childBindings = [childBindings];
            return childBindings.map((cBinding) => toSQL(cBinding, children[0])).join(node.delim ?? "");
        default:
            return "";
    }
}
