import * as duckdb from '@duckdb/duckdb-wasm';
import duckdb_wasm from '@duckdb/duckdb-wasm/dist/duckdb-mvp.wasm?url';
import mvp_worker from '@duckdb/duckdb-wasm/dist/duckdb-browser-mvp.worker.js?url';
import duckdb_wasm_eh from '@duckdb/duckdb-wasm/dist/duckdb-eh.wasm?url';
import eh_worker from '@duckdb/duckdb-wasm/dist/duckdb-browser-eh.worker.js?url';
import { Backend } from "./backend";
export class DuckDBBackend extends Backend {
    name;
    url;
    db = null;
    conn = null;
    constructor(spec) {
        super();
        this.name = spec.name;
        this.url = spec.url;
    }
    async init(type) {
        const MANUAL_BUNDLES = {
            mvp: {
                mainModule: duckdb_wasm,
                mainWorker: mvp_worker,
            },
            eh: {
                mainModule: duckdb_wasm_eh,
                mainWorker: eh_worker,
            },
        };
        // Select a bundle based on browser checks
        const bundle = await duckdb.selectBundle(MANUAL_BUNDLES);
        // Instantiate the asynchronus version of DuckDB-wasm
        const worker = new Worker(bundle.mainWorker);
        const logger = new duckdb.ConsoleLogger();
        this.db = new duckdb.AsyncDuckDB(logger, worker);
        await this.db.instantiate(bundle.mainModule, bundle.pthreadWorker);
        this.conn = await this.db.connect();
        if (this.name && this.url && type === true) {
            await this.loadCSVURL(this.name, this.url);
        }
        else {
            await this.loadCSVString(this.name, this.url);
        }
    }
    async loadCSVString(name, string) {
        await this.db.registerFileText(name, string);
        await this.conn.insertCSVFromPath(name, {
            name,
            schema: "main",
            detect: true,
            header: true
        });
        return this;
    }
    async loadCSVURL(name, url) {
        let response = await fetch(url);
        let buf = await response.arrayBuffer();
        const u8buf = new Uint8Array(buf);
        await this.db.registerFileBuffer(name, u8buf);
        await this.conn.insertCSVFromPath(name, {
            name,
            schema: "main",
            detect: true,
            header: true
        });
        return this;
        //await this.db.insertCSVFromPath(this.conn, this.url, {
        //  schema: 'main',
        //  name: this.name,
        //  detect: true,
        //  header: true
        //});
        //await db.registerFileURL('remote.parquet', 'https://origin/remote.parquet');
    }
    async execute(q, ...args) {
        let res = await this.conn.query(q);
        let rows = res.toArray().map(Object.fromEntries);
        rows.columns = res.schema.fields.map((d) => d.name);
        console.log("duckdb.execute", q, rows, ...args);
        this.emit("data", rows, ...args);
        this.emit("debug", rows);
        return rows;
    }
}
