import * as R from "ramda";
import { ArrayBackend } from "./array";
import { URLBackend } from "./url";
import { DuckDBBackend } from "./duckdb";
import { ClientDiffTreeBackend } from "./clientdt";
import { IdentityBackend } from "./identity";
import { RandomBackend } from "./random";
// add for purposes of FIST
import { QueryBackend } from "./query";
import { OverlayBackend } from "./overlay";
export { ArrayBackend, URLBackend, DuckDBBackend, ClientDiffTreeBackend, IdentityBackend, RandomBackend, QueryBackend, OverlayBackend };
let backendTypes = {
    "ArrayBackend": ArrayBackend,
    "URLBackend": URLBackend,
    "DuckDBBackend": DuckDBBackend,
    "ClientDiffTreeBackend": ClientDiffTreeBackend,
    "IdentityBackend": IdentityBackend,
    "RandomBackend": RandomBackend,
    "QueryBackend": QueryBackend
};
export class BackendLibrary extends Map {
    get backends() {
        return R.values(this);
    }
    // topologically sort backend specs by their 
    // .backend dependencies and instantiate them
    createBackends(specs) {
        for (const spec of specs) {
            this.createBackend(spec);
        }
    }
    // Instantiate a backend object and add to library.
    createBackend(spec) {
        let klass = backendTypes[spec.type];
        if (!klass)
            throw new Error(`Backend type ${spec.type} not found`);
        let backend = new klass(spec);
        this[spec.id] = backend;
        console.info("createBackend", spec.id, backend);
        return backend;
    }
}
