import { writable, get } from "svelte/store";
import * as R from "ramda";
import { viewDomContainers, backendLib, viewLib } from "./stores";
import * as layouts from "./layout/index";
import * as views from "./view/index";
let componentTypes = R.mergeLeft(layouts.componentTypes, views.componentTypes);
function getComponentType(spec) {
    return componentTypes[spec.type] || null;
}
function getComponent(spec, target, onStateUpdate) {
    let constructor = componentTypes[spec?.type];
    if (!constructor) {
        console.log("no constructor for", spec.type);
        return null;
    }
    let component = new constructor({
        target,
        props: { spec }
    });
    component.$on("state", (s) => { onStateUpdate(s.detail); });
    return component;
}
function getComponents(specs, target, onStateUpdate) {
    return (specs || []).map((cspec) => getComponent(cspec, target, onStateUpdate));
}
// XXX: We assume that mappings only renames data
// XXX: We do not support positional attributes because tuples are objects
export function project(data, mappings) {
    if (!mappings)
        return data;
    if (!data)
        return data;
    var entries = Object.entries(mappings);
    let num = /^-?[\d]+$/;
    var f = (o) => {
        var ret = {};
        for (let [tar, src] of entries) {
            if (num.test(o[src]))
                ret[tar] = parseInt(o[src]);
            else
                ret[tar] = o[src];
        }
        return ret;
    };
    return (R.is(Array, data)) ? data.map(f) : f(data);
}
function createViews(viewSpecs, onStateUpdate) {
    let _viewLib = get(viewLib);
    let _backendLib = get(backendLib);
    let _viewDomContainers = get(viewDomContainers);
    return viewSpecs.map((spec) => {
        let view = _viewLib[spec.id];
        if (view)
            return view;
        // XXX: view spec should explicitly specify the ref
        let el = _viewDomContainers[spec.ref ?? spec.id];
        console.log("createView.get Ref Dom", el, spec.id, spec);
        let component = getComponent(spec, el, onStateUpdate);
        console.log("createView", spec.type, _backendLib[spec.backend]);
        // save the component back to writable store
        _viewLib[spec.id] = component;
        viewLib.set(_viewLib);
        // Connect backend to the View
        const backend = _backendLib[spec.backend];
        // handle mapping as postprocessing
        let projection = (t) => project(t, spec.mapping);
        backend?.registerView(component, projection);
        return component;
    });
}
export { EventEmitter } from "./eventemitter";
export { getComponentType, getComponent, getComponents, createViews };
