import { select, selectAll } from "d3-selection";
import * as d3selection from "d3-selection";
import { scaleLinear, scaleOrdinal } from "d3-scale";
import { svg } from "d3-fetch";
export const DEFAULTMARGIN = { top: 30, right: 20, bottom: 30, left: 40 };
export function setupSizes({ fullWidth, fullHeight, margin = DEFAULTMARGIN }) {
    var width = fullWidth - margin.left - margin.right, height = fullHeight - margin.top - margin.bottom;
    return {
        width,
        height
    };
}
export function setupD3({ x, y, // scales 
xAxis, yAxis, xAttr, yAttr, title, name, width, height, margin = DEFAULTMARGIN }) {
    selectAll;
    select;
    d3selection;
    var svg = select(`#${name}`)
        .append("svg")
        .attr("width", width + margin.left + margin.right)
        .attr("height", height + margin.top + margin.bottom)
        .append("g")
        .attr("transform", `translate(${margin.left},${margin.top})`);
    // add axis
    svg.append("g")
        .attr("class", "x axis")
        .attr("transform", "translate(0," + height + ")")
        .call(xAxis)
        .selectAll("text")
        .style("text-anchor", "end")
        .attr("dx", "-0.5em")
        .attr("dy", "0.55em")
        .attr("transform", "rotate(-45)");
    svg.append("g")
        .attr("class", "y axis")
        .call(yAxis)
        .append("text")
        .attr("transform", "rotate(-90)")
        .attr("y", 5)
        .attr("dy", ".71em")
        .style("text-anchor", "end")
        .text(`${yAttr}`);
    svg.append("text")
        .attr("x", width / 2)
        .attr("y", 0 - margin.top / 2)
        .attr("text-anchor", "middle")
        .style("font-size", "16px")
        .style("text-decoration", "underline")
        .text(`${title}`);
    return svg;
}
export function setupGroupedAxes({ width, height }) {
    // for x axis element
    var x0 = scaleOrdinal().rangeRoundBands([0, width], 0.15);
    // for grouped elements
    var x1 = scaleOrdinal();
    var y = scaleLinear().range([height, 0]);
    var colors = ["#46a832", "#3246a8", "#a83242", "#f7c348", "#8924d6", "#000000"];
    var color = scaleOrdinal()
        .range(colors);
    var xAxis = svg.axis().scale(x0).orient("bottom");
    var yAxis = svg.axis().scale(y).orient("left").ticks(10);
    return {
        x0,
        x1,
        y,
        color,
        xAxis,
        yAxis
    };
}
