# Views 

Each view is a Svelte component that has the following structure:

```ts
//
// Passed in when creating component
//
export let spec:ViewSpec;   

//
// Called by application or interaction
//

// Used to register a new interaction that relies
// on this view's manipulation state.
//
// The interaction will specify a manipulation type (i.type)
// that the view can interpret as it wishes.
export function addSourceIact(i:Interaction) {}

// Data that the view uses to render itself, even on initialization,
// is always provided by render()
export function render(table:Array<object>) {}
```



The lifecycle of a view is as follows

* The svelte component is initialized and it creates its DOM elements
* The view's backend is executed and the result is passed to `render()`.   
* The application adds an interaction that uses the view as a source.   The view needs to store the interaction (typically just an Array) and interpret its manipulation type.
* When the user manipulates the view/widget, handle the DOM events, update the view's state, and call the appropriate interactions' `trigger(state)` method.


## JSON Spec

See the [Typescript spec definition](./spec.ts).


